#!/usr/local/bin/bash

##############################################################################
#                                                                            #
#  InstallPackage.sh -- Skript zum Installieren eines Packages von den       #
#                       beiden Package-DVDs fr FreeBSD 5.x.                 #
#                                                                            #
##############################################################################

# Vom Benutzer anzupassende Variablen:
DVDDEVICE=/dev/acd0	# Die Gertedatei des ersten DVD-Laufwerks
SECDVDDEVICE=/dev/acd1	# Die Gertedatei des zweiten DVD-Laufwerks (wenn nur
                        # ein DVD-Laufwerk vorhanden ist, kommentieren Sie
                        # die mit SECDVDDEVICE beginnende Zeile einfach aus!)
MOUNTPOINT=/cdrom	# Der Mountpoint fr die DVD-ROM
SECMOUNTPOINT=/cdrom1	# Der Mountpoint fr die zweite DVD-ROM
TEMPDIR=/tmp/Packages	# Hier werden die bentigten Packages der anderen
                        # DVD temporr zwischengespeichert


##############################################################################
# Ab hier keine weiteren nderungen durch den Benutzer!

# Sind wir root?
if [ "$UID" != "0" ]
   then	echo -e "\nNur der Superuser kann Packages installieren!\n"
	exit 1
fi

if [ -n "$SECDVDDEVICE" ]
   then	######################################################################
	# Fall 1: Es sind ZWEI DVD-Laufwerke im System vorhanden
	echo -e "\nLegen Sie die erste DVD in das Laufwerk $DVDDEVICE"
	echo "und die zweite DVD in das Laufwerk $SECDVDDEVICE ein."
	echo -n "ENTER = Fortsetzen   STRG+C = Abbrechen  "
	read CHOICE

	# Lege bentigte Verzeichnisse bei Bedarf an.
	test -d "$MOUNTPOINT"  ||  mkdir -p "$MOUNTPOINT"  ||  exit 1
	test -d "$SECMOUNTPOINT"  ||  mkdir -p "$SECMOUNTPOINT"  ||  exit 1
	test -d "$TEMPDIR"  ||  mkdir -p "$TEMPDIR"  ||  exit 1

	echo -e "\nMounte $DVDDEVICE auf $MOUNTPOINT und $SECDVDDEVICE auf $SECMOUNTPOINT..."
	mount_cd9660 "$DVDDEVICE" "$MOUNTPOINT"  ||  exit 1
	mount_cd9660 "$SECDVDDEVICE" "$SECMOUNTPOINT"  ||  exit 1

	echo -e "\nAuf welcher DVD ist das zu installierende Package vorhanden?"
	echo -n "1 = DVD Nr. 1   2 = DVD Nr. 2   andere Eingabe = Abbrechen   "
	read DVDNUMBER

	case "$DVDNUMBER" in
	  1)  cd "$MOUNTPOINT"        ;;
	  2)  cd "$SECMOUNTPOINT"     ;;
	  *)  cd
              umount "$MOUNTPOINT"
              umount "$SECMOUNTPOINT"
              exit 1                  ;;
	esac

	echo -e "\nZu installierendes Package (Tab-Taste zur Vervollstndigung):"
	read -e REQ_PACKAGE

	if [ ! -f "$REQ_PACKAGE" ]
           then echo -e "\n$REQ_PACKAGE existiert nicht!\n"
                cd
                umount "$MOUNTPOINT"
                umount "$SECMOUNTPOINT"
                exit 1
        fi

        PKG_PATH="$MOUNTPOINT:$SECMOUNTPOINT" pkg_add "$REQ_PACKAGE"

	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT und $SECDVDDEVICE auf $SECMOUNTPOINT aus..."
	cd
	umount "$MOUNTPOINT"
        umount "$SECMOUNTPOINT"

   else	######################################################################
	# Fall 2: Es ist nur   E I N   DVD-Laufwerk im System vorhanden

	# Zuerst ist die DVD einzulegen, auf der sich das zu installierende
	# Package befindet.
	echo -e "\nDVD einlegen, auf der sich das zu installierende Package befindet."
	echo -n "ENTER = Fortsetzen   STRG+C = Abbrechen   "
	read CHOICE

	# Lege bentigte Verzeichnisse bei Bedarf an.
	test -d "$MOUNTPOINT"  ||  mkdir -p "$MOUNTPOINT"  ||  exit 1
	test -d "$TEMPDIR"  ||  mkdir -p "$TEMPDIR"  ||  exit 1

	# Mounten der DVD und wechseln in den Mountpoint.
	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT ein..."
	mount_cd9660 "$DVDDEVICE" "$MOUNTPOINT"  ||  exit 1
	cd "$MOUNTPOINT"

	# Frage den Benutzer, welches Package installiert werden soll:
	echo -e "\nZu installierendes Package (Tab-Taste zur Vervollstndigung):"
	read -e REQ_PACKAGE

	# Wenn das Package nicht auffindbar ist, brechen wir sofort ab!
	if [ ! -f "$REQ_PACKAGE" ]
           then echo -e "\n$REQ_PACKAGE existiert auf dieser DVD nicht!\n"
                cd
                umount "$MOUNTPOINT"
                exit 1 
        fi

	# Versuche, das Package zu installieren. Wenn Packages fehlen, sind
	# diese vermutlich auf der anderen DVD. Wir speichern die Namen der
        # fehlenden Packages zunchst in $MISSED_LIST (der Trenner ist laut
        # $IFS der Zeilenumbruch!)
	echo -e "\nVersuche, $REQ_PACKAGE zu installieren..."
        IFS="
"
        for LINE in $(pkg_add "$REQ_PACKAGE" 2>&1  |  \
                      grep "could not find package")
	do
	  MISSED_PACKAGE="$(echo $LINE  |  cut -d\  -f6)"
	  MISSED_LIST="$MISSED_LIST
$MISSED_PACKAGE"
	  echo "===>  $MISSED_PACKAGE ist auf dieser DVD nicht vorhanden."
	done

	# Entweder ist der Vorgang jetzt erledigt oder wir brauchen die nchste
	# DVD. Fr beides mu die jetzige DVD ausgehngt werden:
	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT aus..."
	cd
	umount "$MOUNTPOINT"  ||  exit 1

	# Wenn keine Packages gefehlt haben, ist die Sache fr uns erledigt.
	if [ -z "$MISSED_LIST" ]
           then echo -e "\n$REQ_PACKAGE wurde installiert.\n"
                exit 0 
        fi

	# Nun fordern wir uns die andere DVD an, um dort nach den fehlenden
	# Packages zu suchen:
	echo -e "\nLegen Sie nun die andere der beiden DVDs in das Laufwerk ein."
	echo -n "ENTER = Fortsetzen   STRG+C = Abbrechen   "
	read CHOICE

	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT ein...\n"
	mount_cd9660 "$DVDDEVICE" "$MOUNTPOINT"  ||  exit 1
	cd "$MOUNTPOINT"

	# Wir versuchen, jedes der fehlenden Packages von der eingelegten DVD
	# in das temporre Verzeichnis zu kopieren.
	for MISSED_PACKAGE in $MISSED_LIST
	do
	  echo "Kopiere $MISSED_PACKAGE.tbz nach $TEMPDIR..."
	  cp "$MISSED_PACKAGE".tbz "$TEMPDIR"  ||  exit 1
	done

	# Wir brauchen nun wieder die DVD, auf der sich das zu installierende
	# Package befindet. Dazu mu zunchst die aktuelle DVD ausgehngt
	# werden:
	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT aus..."
	cd
	umount "$MOUNTPOINT"  ||  exit 1

	# Jetzt fordern wir uns wieder die DVD mit dem zu installierenden
	# Package an.
	echo -e "\nLegen Sie nun wieder die DVD ein, auf der sich $REQ_PACKAGE befindet."
	echo -n "ENTER = Fortsetzen   STRG+C = Abbrechen   "
	read CHOICE

	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT ein..."
	mount_cd9660 "$DVDDEVICE" "$MOUNTPOINT"  ||  exit 1
	cd "$MOUNTPOINT"

	# Nun knnen wir das Package installieren, in dem wir den $PKG_PATH
	# auf das temporre Verzeichnis (enthlt die bentigten Packages von
	# der anderen DVD) sowie auf das aktuelle Verzeichnis (= die aktuell
	# eingelegte DVD mit dem zu installierenden Package sowie evtl.
	# anderen bentigten Packages) setzen und pkg_add(1) aufrufen:
	echo -e "\nInstalliere $REQ_PACKAGE..."
	PKG_PATH="$TEMPDIR:$(pwd)" pkg_add "$REQ_PACKAGE"

	echo -e "\nHnge $DVDDEVICE auf $MOUNTPOINT aus..."
	cd
	umount "$MOUNTPOINT"

	# Damit die Festplatte nicht vollmllt, lschen wir die in $MISSED_LIST
	# aufgefhrten Packages wieder aus dem temporren Verzeichnis. Wir
	# lschen nicht einfach das ganze temporre Verzeichnis, weil wir
	# sonst mglicherweise "Unschuldige" mit ins Nirvana reien!
	echo -e "\nRume $TEMPDIR auf..."
	for MISSED_PACKAGE in $MISSED_LIST
	do
	  rm -f "$TEMPDIR"/"$MISSED_PACKAGE".tbz
	done
fi

echo ; exit
